# Copyright (C) 2020 by ÿnérant, eichhornchen, nicomarg, charlse
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest
from random import randint

from squirrelbattle.interfaces import Map, Tile
from squirrelbattle.mapgeneration import randomwalk, broguelike

def is_connex(grid):
    h, w = len(grid), len(grid[0])
    y, x = randint(0, h-1), randint(0, w-1)
    while not(grid[y][x].can_walk()):
        y, x = randint(0, h-1), randint(0, w-1)
    queue = Map.neighbourhood(grid, y, x)
    while queue != []:
        y, x = queue.pop()
        if grid[y][x].can_walk():
            grid[y][x] = Tile.WALL
            queue += Map.neighbourhood(grid, y, x)
    return not(any([any([t.can_walk() for t in l]) for l in grid]))

class TestRandomWalk(unittest.TestCase):
    def setUp(self) -> None:
        #we set no_lone_walls to true for 100% coverage
        params = randomwalk.DEFAULT_PARAMS
        params["no_lone_walls"] = True
        self.generator = randomwalk.Generator(params = params)

    def test_starting(self) -> None:
        """
        Create a map and check that the whole map is accessible from the starting position using a 
        depth-first search
        """
        m = self.generator.run()
        self.assertTrue(m.tiles[m.start_y][m.start_x].can_walk())

    def test_connexity(self) -> None:
        m = self.generator.run()
        self.assertTrue(is_connex(m.tiles))

class TestBroguelike(unittest.TestCase):
    def setUp(self) -> None:
        self.generator = broguelike.Generator()
    
    def test_connexity(self) -> None:
        m = self.generator.run()
        self.assertTrue(is_connex(m.tiles))
